#!/bin/bash

# Set up the CMSSW environment
cd /afs/cern.ch/user/a/akub19/scratch0/AutoVal/CMSSW_2_1_10/src
eval `scramv1 runtime -sh`
cd RecoLocalMuon/CSCValidation/test


# dataset to run over
#DATASET=/BeamHalo/BeamCommissioning08-PromptReco-v1/RECO
DATASET=/Cosmics/Commissioning08-v1/RAW
# globaltag to use
GLOBALTAG=CRAFT_V3P::All
# number of events to run over per run
NUMEVENTS=200000
# CMSSW release
RELEASE=`echo $CMSSW_VERSION`

DATE1=`date '+%Y-%m-%d' --date='1 days ago'`         # yesterday's date for DBS query
#DATE1="2008-10-24"
DATE2=`date '+%m-%d-%Y' --date='1 days ago'`         # yesterday's date for web folder name
#DATE2="10-24-2008"

# Logfile for debugging
LOGFILE=${DATE2}.log

# Make web directory and directory in personal area to store output root files (can get big)
mkdir ${DATE2}
rfmkdir /castor/cern.ch/user/a/akub19/CSCValRootFiles/${DATE2}


#First, query DBS for run numbers taken today
#There needs to be some other criteria to select appropriate runs
DBSFAILRUNS=1
NTRYRUNS=0
while [ ${DBSFAILRUNS} -ne 0 ]
do
    if [ ${NTRYRUNS} -lt 30 ]
    then
        `./dbsquery.py --input="find run where run.createdate = ${DATE1} and dataset = ${DATASET}" --limit=-1 > runlist_${DATE2}.txt`
        # ok, this is dumb, but look for word Traceback in runlist, if it is there it means the query failed, so try it again
        DBSFAILRUNS=`awk '/'Traceback'/ {print;exit}' runlist_${DATE2}.txt | wc -w`
        NTRYRUNS=`expr ${NTRYRUNS} + 1`
    fi
done

# if DBS query failed a lot, log it
if [ ${DBSFAILRUNS} -gt 0 ]
then
     echo "DBS query for list of run numbers failed 30 times..." >> ${LOGFILE}
fi

# if DBS query was successful continue
if [ ${DBSFAILRUNS} -eq 0 ]
then
   # Loop over run numbers in runlist
   # Maybe see if the runlist makes sense first?
   NLINES=`cat runlist_${DATE2}.txt | wc -l`
   NLINES=`expr ${NLINES} - 1`
   COUNT=4
   while [ ${COUNT} -le ${NLINES} ]
   do
       # take run in runlist, get rid of space at the end
       RUN=`head runlist_${DATE2}.txt -n ${COUNT} | tail -1`
       RUN=`echo ${RUN} | sed 's/[ \t]*$//'`

       # Query DBS for a list of files froma certain dataset and run number
       # Need error checking here also
       FILELIST=filelist_run${RUN}.txt
       DBSFAILFILE=1
       NTRYFILE=0
       # again do some dumb error checking
       while [ ${DBSFAILFILE} -ne 0 ]
       do
           if [ ${NTRYFILE} -lt 30 ]
           then
               `./dbsquery.py --input="find file where dataset=${DATASET} and run=${RUN}" --cff --limit=250 > ${FILELIST}`
               DBSFAILFILE=`awk '/'Traceback'/ {print;exit}' ${FILELIST} | wc -w`
               NTRYFILE=`expr ${NTRYFILE} + 1`
           fi
       done

       if [ ${DBSFAILFILE} -gt 0 ]
       then
           echo "DBS query for list of files failed 30 times for run ${RUN}..." >> ${LOGFILE}
       fi

       # Print out the python config file which will be used to run
       CONFIGFILE=runvalidation_run${RUN}_cfg.py
       cat >> ${CONFIGFILE}<<EOF2
import FWCore.ParameterSet.Config as cms

process = cms.Process("TEST")

process.load("Configuration/StandardSequences/Geometry_cff")
process.load("Configuration/StandardSequences/MagneticField_cff")
process.load("Configuration/StandardSequences/FrontierConditions_GlobalTag_cff")
process.load("Configuration/StandardSequences/RawToDigi_Data_cff")
process.load("Configuration.StandardSequences.Reconstruction_cff")
process.GlobalTag.globaltag = '${GLOBALTAG}'

process.source = cms.Source("PoolSource",
    fileNames = cms.untracked.vstring(
EOF2

       # Insert the filelist into the config file
       # DBS Query has annoying header and some other unwanted stuff, so delete these lines
       sed -e '1,4d' -e 's/}.*//' -e '/^$/d' ${FILELIST} >> ${CONFIGFILE}

       cat >> ${CONFIGFILE}<<EOF3
)
)

process.maxEvents = cms.untracked.PSet(
    input = cms.untracked.int32(${NUMEVENTS})
)

process.cscValidation = cms.EDFilter("CSCValidation",
    rootFileName = cms.untracked.string('validationHists_run${RUN}.root'),
    isSimulation = cms.untracked.bool(False),
    writeTreeToFile = cms.untracked.bool(True),
    useDigis = cms.untracked.bool(True),
    detailedAnalysis = cms.untracked.bool(False),
    stripDigiTag = cms.InputTag("muonCSCDigis","MuonCSCStripDigi"),
    wireDigiTag = cms.InputTag("muonCSCDigis","MuonCSCWireDigi"),
    compDigiTag = cms.InputTag("muonCSCDigis","MuonCSCComparatorDigi"),
    cscRecHitTag = cms.InputTag("csc2DRecHits"),
    cscSegTag = cms.InputTag("cscSegments"),
    saMuonTag = cms.InputTag("cosmicMuonsEndCapsOnly"),
    l1aTag = cms.InputTag("gtDigis"),
    simHitTag = cms.InputTag("g4SimHits", "MuonCSCHits")
)

process.p = cms.Path(process.muonCSCDigis*process.csc2DRecHits*process.cscSegments*process.cscValidation)
EOF3

       # run CSCValidation
       cmsRun ${CONFIGFILE}

       # make a subdirectory for each run
       cd ${DATE2}
       mkdir run${RUN}
       cd ..
   
       # run the root macro which formats and prints select plots in .png format
       ./makePlots.sh validationHists_run${RUN}.root
       mkdir PNGS
       mv *.png PNGS

       # move the root file to somewhere with plenty of room
       rfcp validationHists_run${RUN}.root /castor/cern.ch/user/a/akub19/CSCValRootFiles/${DATE2}
       rm validationHists_run${RUN}.root

       # create a front page for each run and cp html to run directory
       ./writeSummaryHTML.sh ${RUN} ${RELEASE} ${DATE2} ${GLOBALTAG} ${NUMEVENTS} ${DATASET}
       cp -r web/* ${DATE2}/run${RUN}
       mv Summary.html ${DATE2}/run${RUN}/Site
       mv PNGS ${DATE2}/run${RUN}/Site 
       mv ${CONFIGFILE} ${DATE2}/run${RUN}/Site
       mv ${FILELIST} ${DATE2}/run${RUN}
       mv deadchamberlist.txt ${DATE2}/run${RUN}/Site
       COUNT=`expr ${COUNT} + 1`  

    done

    # keep a record of the run list for debugging
    mv runlist_${DATE2}.txt ${DATE2}
    # tar the results
    tar -czvf ${DATE2}.tgz ${DATE2}
    #rm -r ${DATE2}
fi
